﻿#ifndef zego_api_edu_whiteboard_constant_h
#define zego_api_edu_whiteboard_constant_h

#include "zego-api-edu-defines.h"

/** 白板id类型，唯一标识一块白板 */
typedef unsigned long long ZegoWhiteboardId;

/** 白板画布上图元id类型，唯一标识一个图元 */
typedef unsigned long long ZegoWhiteboardGraphicId;


/** 白板模式 */
enum ZegoWhiteboardMode
{
	kZegoWhiteboardModeFullUpdate = 0x1,            /**< 单个图元绘制完毕后才全量同步该图元 */
	kZegoWhiteboardModeRealTime = 0x2,            /**< 实时模式，边绘制边同步，适用于人数较少的环境 */
	kZegoWhiteboardModeSingleOperator = 0x3,            /**< 1对多非互动模式 */
};

/** 白板图片填充模式 */
enum ZegoWhiteboardViewImageFitMode {
	kZegoWhiteboardViewImageFitModeLeft = 0,         /**< 靠左对齐按宽或按高等比放大或缩小 */
	kZegoWhiteboardViewImageFitModeRight,           /**< 靠右对齐按宽或按高等比放大或缩小 */
	kZegoWhiteboardViewImageFitModeBottom,          /**< 底部对齐按宽或按高等比放大或缩小 */
	kZegoWhiteboardViewImageFitModeTop,             /**< 顶部对齐按宽或按高等比放大或缩小 */
	kZegoWhiteboardViewImageFitModeCenter           /**< 居中对齐按宽或按高等比放大或缩小 */
};


/** 白板画布上简单图元类型 */
enum ZegoWhiteboardGraphic
{
	kZegoWhiteboardGraphicNone = 0x0,             /**< 空图元 0 */
	kZegoWhiteboardGraphicPath = 0x1,             /**< 涂鸦  1*/
	kZegoWhiteboardGraphicText = 0x2,             /**< 文本  2*/
	kZegoWhiteboardGraphicLine = 0x4,             /**< 直线  4*/
	kZegoWhiteboardGraphicRect = 0x8,             /**< 矩形  8*/
	kZegoWhiteboardGraphicEllipse = 0x10,            /**< 圆或椭圆  16*/
	kZegoWhiteboardGraphicLaser = 0x80,            /**< 激光笔  128*/
	kZegoWhiteboardGraphicImage = 0x100,				/**< 图片 256*/
	kZegoWhiteboardGraphicBackGround = 0x200,     /**< 背景图 512*/     //新增设置背景图片
	kZegoWhiteboardGraphicRealTimePath = 999,             /**< 实时涂鸦  999*/
};

enum ZegoWhiteboardGraphicImageType {
	//图片图元
	ZegoWhiteboardGraphicImageGraphic = 0,
	//自定义教具图片
	ZegoWhiteboardGraphicImageCustom = 1
};

/** 白板画布上简单图元粗细（在画布视口宽度为1280px情况下的推荐标准值，画布大小变化时请自行缩放图元） */
enum ZegoWhiteboardGraphicSizeLevel
{
	kZegoWhiteboardGraphicSizeLevel1 = 2,               /**< 单位px */
	kZegoWhiteboardGraphicSizeLevel2 = 4,
	kZegoWhiteboardGraphicSizeLevel3 = 10,
	kZegoWhiteboardGraphicSizeLevel4 = 20,
};

/** 白板画布上文本图元字体大小（在画布视口宽度为1280px情况下的推荐标准值，画布大小变化时请自行缩放图元） */
enum ZegoWhiteboardGraphicFontLevel
{
	kZegoWhiteboardGraphicFontLevel1 = 16,              /**< 单位px */
	kZegoWhiteboardGraphicFontLevel2 = 24,
	kZegoWhiteboardGraphicFontLevel3 = 36,
	kZegoWhiteboardGraphicFontLevel4 = 48,
};

/** 白板画布滑动、滚动方向 */
enum ZegoWhiteboardCanvasScrollDirection
{
	kZegoWhiteboardCanvasScrollDirectionHorizontal = 0x1,        /**< 水平方向 */
	kZegoWhiteboardCanvasScrollDirectionVertical = 0x2,        /**< 竖直方向 */
};

/**
* 图片类型枚举定义
*/
enum ZegoWhiteboardViewImageType 
{											
	ZegoWhiteboardViewImageGraphic = 0,		/**< 图片图元 */
	ZegoWhiteboardViewImageCustom = 1		/**< 自定义教具图片 */
};


/**
白板的图元操作权限类型
*/
enum ZegoGraphicOperateType
{
	kZegoGraphicOperateTypeUpdate = 0x2,	//编辑
	kZegoGraphicOperateTypeDelete = 0x4,    //删除
	kZegoGraphicOperateTypeMove = 0x8,      //移动
	kZegoGraphicOperateTypeClear = 0x10,    //清空
	kZegoGraphicOperateTypeCreate = 0x20,   //创建
};

/**
白板的操作权限类型
*/
enum ZegoWhiteboardOperateType
{
	kZegoWhiteboardOperateTypeScale = 1,  	//缩放
	kZegoWhiteboardOperateTypeScroll = 2,   //滚动
};
#endif
