#ifndef ZEGO_WHITEBOARD_CANVAS_H
#define ZEGO_WHITEBOARD_CANVAS_H

#include "zego-whiteboard-defines.h"

#ifdef __cplusplus
extern "C" {
#endif

	/**
	 * 设置使用的白板工具
	 *
	 * @param type 白板工具类型
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_tool_type(enum ZegoWhiteboardToolType type);

	/**
	 * 获取正在使用的白板工具
	 *
	 * @return 正在使用的白板工具类型
	 */
	ZEGOAPI enum ZegoWhiteboardToolType ZEGOCALL zego_whiteboard_get_tool_type();

	/**
	 * 设置画笔颜色
	 *
	 * @param argb 画笔颜色
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_brush_color(unsigned int argb);

	/**
	 * 获取当前画笔颜色
	 *
	 * @return 当前白板画笔颜色
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_get_brush_color();

	/**
	 * 设置画笔粗细
	 *
	 * 绘制的画笔粗细并不是设置的值，而是根据白板实际的显示大小作调整，实际绘制的size=白板view宽度/1280*size
	 *
	 * @param size 画笔粗细
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_brush_size(unsigned int size);

	/**
	 * 获取当前画笔粗细
	 *
	 * @return 当前白板画笔粗细
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_get_brush_size();

	/**
	 * 设置文本大小
	 *
	 * 绘制的文本大小并不是设置的值，而是根据白板实际的显示大小作调整，实际绘制的size=白板view宽度/1280*size
	 *
	 * @param size 文本大小
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_text_size(unsigned int size);

	/**
	 * 获取当前文本大小
	 *
	 * @return 当前文本大小
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_get_text_size();

	/**
	 * 设置文本是否粗体
	 *
	 * @param bold 是否粗体
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_text_bold(bool bold);
	 
	/**
	 * 获取文本是否粗体
	 *
	 * @return 是否粗体
	 */
	ZEGOAPI bool ZEGOCALL zego_whiteboard_get_text_bold();

	/**
	 * 设置文本是否斜体
	 *
	 * @param italic 是否斜体
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_text_italic(bool italic);

	/**
	 * 获取文本是否斜体
	 *
	 * @return 是否斜体
	 */
	ZEGOAPI bool ZEGOCALL zego_whiteboard_get_text_italic();

	/**
	 * 设置白板是否允许操作
	 *
	 * 白板默认允许操作，禁用后，白板不响应鼠标消息
	 *
	 * @param whiteboard_id 白板标识
	 * @param enable 是否允许操作
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_enable(ZegoWhiteboardID whiteboard_id, bool enable);

	/**
	 * 获取白板是否允许操作
	 *
	 * @return 是否允许操作
	 */
	ZEGOAPI bool ZEGOCALL zego_whiteboard_is_enable(ZegoWhiteboardID whiteboard_id);

	/**
	 * 设置白板背景色
	 *
	 * @param whiteboard_id 白板标识
	 * @param a_rgb 白板背景色
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_background_color(ZegoWhiteboardID whiteboard_id, unsigned int argb);

	/**
	 * 获取白板背景色
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 白板背景色
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_get_background_color(ZegoWhiteboardID whiteboard_id);

	/**
	 * 撤销白板操作
	 *
	 * 只能撤销本地的操作
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_undo(ZegoWhiteboardID whiteboard_id);

	/**
	 * 重做白板操作
	 *
	 * 只能重做本地操作
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_redo(ZegoWhiteboardID whiteboard_id);

	/**
	 * 清空白板所有内容
	 *
	 * 清空操作无法撤销
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_clear(ZegoWhiteboardID whiteboard_id);

#ifdef __cplusplus
}
#endif


#endif