#ifndef ZEGO_WHITEBOARD_CONSTANT_H
#define ZEGO_WHITEBOARD_CONSTANT_H

#ifdef WIN32
#ifdef ZEGO_SDK_EXPORTS
#define ZEGOAPI __declspec(dllexport)
#define ZEGOCALL __cdecl
#elif defined ZEGO_SDK_STATIC
#define ZEGOAPI
#define ZEGOCALL
#else
#define ZEGOAPI __declspec(dllimport)
#define ZEGOCALL __cdecl
#endif
#else
#define ZEGOAPI __attribute__((visibility("default")))
#define ZEGOCALL
#endif

#define WINDOW_HANDLE void*
typedef unsigned int ZegoSeq;
typedef unsigned long long ZegoWhiteboardViewID;
typedef unsigned long long ZegoWhiteboardID;

/**
 * 错误码
 *
 */
enum ZegoWhiteboardErrorCode
{
	/**
	 * 成功
	 */
	kZegoWhiteboardSuccess = 0,

	/**
	 * SDK未初始化
	 */
	kZegoWhiteboardNotInit = 1,

	/**
	 * 文件模块（ZegoDocsView.dll）不存在，无法使用文件功能
	 */
	kZegoDocsModuleNotExist = 10,

	/**
	 * 白板模块（Zegoliveroom.dll或ZegoExpressEngine.dll）不存在，无法使用白板功能
	 */
	kZegoWhiteboardModuleNotExist = 11,

	/**
	 * 文件模块和白板模块都不存在，SDK无法使用
	 */
	kZegoWhiteboardUnavailable = 12,

	/**
	 * 文件不存在
	 */
	kZegoWhiteboardErrorFileNotExist = 2010001,

	/**
	 * 文件被加密
	 */
	kZegoWhiteboardErrorFileEncrypt = 2020001,

	/**
	 * 文件内容过大
	 */
	kZegoWhiteboardErrorFileSizeLimit = 2020002,

	/**
	 * Excel文件sheet过多
	 */
	kZegoWhiteboardErrorFileSheetLimit = 2020003,

	/**
	 * 格式转换失败
	 */
	kZegoWhiteboardErrorConvertFail = 2020004,

	/**
	 * 格式转换被取消
	 */
	kZegoWhiteboardErrorConvertCancel = 2020005,

	/**
	 * 认证参数错误
	 */
	kZegoWhiteboardErrorAuthParamInvalid = 2030001,

	/**
	 * 路径权限不足
	 */
	kZegoWhiteboardErrorFilePathNotAccess = 2030002,

	/**
	 * 初始化失败
	 */
	kZegoWhiteboardErrorInitFailed = 2030003,

	/**
	 * 无法获取当前view宽高
	 */
	kZegoWhiteboardErrorSizeInvalid = 2030004,

	/**
	 * 本地空间不足
	 */
	kZegoWhiteboardErrorFreeSpaceLimit = 2030005,

	/**
	 * 不支持文件上传功能
	 */
	kZegoWhiteboardErrorUploadNotSupported = 2030006,

	/**
	 * 正在上传相同文件
	 */
	kZegoWhiteboardErrorUploadDuplicated = 2030007,

	/**
	 * 内部错误
	 */
	kZegoWhiteboardErrorInternal = 3000001,

	/**
	 * 参数错误
	 */
	kZegoWhiteboardErrorParamInvalid = 3000002,

	/**
	 * 网络超时
	 */
	kZegoWhiteboardErrorNetworkTimeout = 3000003,

	/**
	 * 网络断开
	 */
	kZegoWhiteboardErrorNetworkDisconnect = 3000004,
	
	/**
	 * 网络回包错误
	 */
	kZegoWhiteboardErrorInvalidRsp = 3000005,
	
	/**
	 * 请求过于频繁
	 */
	kZegoWhiteboardErrorRequestTooMany = 3000006,
	
	/**
	 * 未登录房间
	 */
	kZegoWhiteboardErrorNoLoginRoom = 3010001,
	
	/**
	 * 用户不存在
	 */
	kZegoWhiteboardErrorUserNotExist = 3010002,
	
	/**
	 * 白板view不存在
	 */
	kZegoWhiteboardErrorViewNotExist = 3020001,
	
	/**
	 * 创建白板view失败
	 */
	kZegoWhiteboardErrorViewCreateFail = 3020002,
	
	/**
	 * 修改白板view失败
	 */
	kZegoWhiteboardErrorViewModifyFail = 3020003,
	
	/**
	 * 白板view名称过长，最长限制为128字节
	 */
	kZegoWhiteboardErrorViewNameLimit = 3020004,
	
	/**
	 * 白板view的parent不存在
	 */
	kZegoWhiteboardErrorViewParentNotExist = 3020005,
	
	/**
	 * 超过白板最大数量限制
	 */
	kZegoWhiteboardErrorViewNumLimit = 3020006,
	
	/**
	 *动画信息过长
	 */
	ZegoWhiteboardViewErrorAnimationInfoLimit = 3020007,

	/**
	 * 图元不存在
	 */
	kZegoWhiteboardErrorGraphicNotExist = 3030001,
	
	/**
	 * 创建图元错误
	 */
	kZegoWhiteboardErrorGraphicCreateFail = 3030002,
	
	/**
	 * 修改图元错误
	 */
	kZegoWhiteboardErrorGraphicModifyFail = 3030003,
	
	/**
	 * 未开启绘制
	 */
	kZegoWhiteboardErrorGraphicUnableDraw = 3030004,
	
	/**
	 * 单个图元数据大小超过限制
	 */
	kZegoWhiteboardErrorGraphicDataLimit = 3030005,
	
	/**
	 * 超过图元最大数量限制
	 */
	kZegoWhiteboardErrorGraphicNumLimit = 3030006,
	
	/**
	 * 文本字数超过上限
	 */
	kZegoWhiteboardErrorGraphicTextLimit = 3030007,

	/**
	 * 创建白板失败
	 */
	kZegoWhiteboardErrorCreateFail = 3040003,
	
	/**
	 * 销毁白板失败
	 */
	kZegoWhiteboardErrorDestroyFail = 3040004,
	
	/**
	 * attach白板失败
	 */
	kZegoWhiteboardErrorAttachFail = 3040005,
	
	/**
	 * 清空白板失败
	 */
	kZegoWhiteboardErrorClearFail = 3040006,
	
	/**
	 * 滚动白板失败
	 */
	kZegoWhiteboardErrorScrollFail = 3040007,
	
	/**
	 * 撤销操作失败
	 */
	kZegoWhiteboardErrorUndoFail = 3040008,
	
	/**
	 * 重做操作失败
	 */
	kZegoWhiteboardErrorRedoFail = 3040009,
};

/**
 * 文件上传状态
 *
 */
enum ZegoWhiteboardDocsUploadStatus
{
	/**
	 * 文件上传中
	 */
	kZegoWhiteboardDocsUploading = 0,

	/**
	 * 文件转码中
	 */
	kZegoWhiteboardDocsConvering = 1
};

/**
 * 文件转码类型
 *
 * 普通文件转码推荐选择kZegoWhiteboardDocsRenderTypeVectorAndIMG，动态PPT文件转码推荐选择kZegoWhiteboardDocsRenderTypeDynamicPPTH5
 *
 */
enum ZegoWhiteboardDocsRenderType
{
	/**
	 * 向量模式，适用于文件共享SDK PC端、移动端做的文档智能转换，能够更好地支持文档浏览以及更清晰的缩放。
	 */
	kZegoWhiteboardDocsRenderTypeVector = 1,

	/**
	 * 图片模式，适用于文件共享SDK Web端、小程序使用的文档转换，按文件样式每一页都生成图片样式。
	 */
	kZegoWhiteboardDocsRenderTypeIMG = 2,

	/**
	 * 向量和图片模式，适用于当存在Web端、小程序、移动端、PC端，多端的并存情况，在web端、小程序将使用IMG图片模式，而PC端、移动端将使用Vector向量模式。
	 */
	kZegoWhiteboardDocsRenderTypeVectorAndIMG = 3,

	/**
	 * 动态PPT模式，适用于文件共享SDK PC端、移动端、Web端、小程序使用的PPT文件转码格式，会将PPT文件转成动态PPT的H5文件。
	 */
	kZegoWhiteboardDocsRenderTypeDynamicPPTH5 = 6
};

/**
 * 文件类型
 *
 */
enum ZegoWhiteboardDocsType
{
	/**
	 * 未知类型
	 */
	kZegoWhiteboardDocsTypeUnknown = 0x0,

	/**
	 * PPT
	 */
	kZegoWhiteboardDocsTypePPT = 0x1,

	/**
	 * Word
	 */
	kZegoWhiteboardDocsTypeDOC = 0x2,

	/**
	 * Excel
	 */
	kZegoWhiteboardDocsTypeELS = 0x4,

	/**
	 * PDF
	 */
	kZegoWhiteboardDocsTypePDF = 0x8,

	/**
	 * 常见的图片类型
	 */
	kZegoWhiteboardDocsTypeIMG = 0x10,

	/**
	 * 普通文本文件
	 */
	kZegoWhiteboardDocsTypeTXT = 0x20,

	/**
	 * H5
	 */
	kZegoWhiteboardDocsTypeH5 = 0x40,

	/**
	 * H5 PDF
	 */
	kZegoWhiteboardDocsTypeH5PDF = 0x80,

	/**
	 * 图片类型的PDF
	 */
	kZegoWhiteboardDocsTypeIMGAndPDF = 0x100,

	/**
	 * 动态PPT
	 */
	kZegoWhiteboardDocsTypeDynamicPPTH5 = 0x200,

	/**
	 * 文件夹
	 */
	kZegoWhiteboardDocsTypeFolder = 0x10000000,
};

/**
 * 白板类型
 *
 */
enum ZegoWhiteboardRenderMode
{
	/**
	 * 普通白板类型
	 */
	kZegoWhiteboardRenderModeWhiteboard,

	/**
	 * 文件白板类型
	 */
	kZegoWhiteboardRenderModeWhiteboardDocs,
};

/**
 * 白板关联文件信息
 *
 */
struct ZegoWhiteboardViewFileInfo
{
	/**
	 * 文件ID，如果为空，表示没有关联文件
	 */
	char file_id[256];
	
	/**
	 * 文件名称，如果是Excel类型的文件，则为Sheet的名称
	 */
	char file_name[256];
	
	/**
	 * 文件共享鉴权字段，暂未使用
	 */
	char auth_key[256];
	
	/**
	 * 文件类型
	 */
	enum ZegoWhiteboardDocsType file_type;
};

/**
 * 白板属性
 *
 */
struct ZegoWhiteboardViewModel
{
	/**
	 * 白板类型
	 */
	ZegoWhiteboardRenderMode mode;
	
	/**
	 * 白板所属房间id
	 */
	char room_id[256];
	
	/**
	 * 白板名称，当白板类型是文件白板时，则为文件名称
	 */
	char name[128];
	
	/**
	 * 白板的等比宽
	 */
	unsigned int aspect_width;
	
	/**
	 * 白板的等比高
	 */
	unsigned int aspect_height;
	
	/**
	 * 白板的页数
	 */
	int page_count;
	
	/**
	 * 白板的创建时间，单位毫秒
	 */
	unsigned long long create_time;
	
	/**
	 * 白板关联的文件信息，当白板类型是文件白板时，此结构必须填充
	 */
	struct ZegoWhiteboardViewFileInfo file_info;
};

/**
 * 白板工具
 *
 */
enum ZegoWhiteboardToolType
{
	/**
	 * 拖拽工具
	 */
	kZegoWhiteboardToolNone = 0x00,
	
	/**
	 * 涂鸦
	 */
	kZegoWhiteboardToolPen = 0x01,
	
	/**
	 * 文本
	 */
	kZegoWhiteboardToolText = 0x02,
	
	/**
	 * 直线
	 */
	kZegoWhiteboardToolLine = 0x04,
	
	/**
	 * 空心椭圆
	 */
	kZegoWhiteboardToolEllipse = 0x08,
	
	/**
	 * 空心矩形
	 */
	kZegoWhiteboardToolRect = 0x10,
	
	/**
	 * 选择
	 */
	kZegoWhiteboardToolSelector = 0x20,
	
	/**
	 * 橡皮擦
	 */
	kZegoWhiteboardToolEraser = 0x40,
	
	/**
	 * 激光笔
	 */
	kZegoWhiteboardToolLaser = 0x80,
	
	/**
	 * 点击
	 */
	kZegoWhiteboardToolClick = 0x100,
};


#endif
