#ifndef ZEGO_WHITEBOARD_DEFINES_H
#define ZEGO_WHITEBOARD_DEFINES_H

#include "zego-whiteboard-constant.h"

/**
 * SDK运行错误通知
 *
 * @param error_code 错误码
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_error_notify)(ZegoWhiteboardErrorCode error_code, void* user_context);

/**
 * 初始化SDK的回调
 *
 * @param error_code 错误码
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_initialize_callback)(ZegoWhiteboardErrorCode error_code, void* user_context);

/**
 * 反初始化SDK的回调
 *
 * @param error_code 错误码
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_uninitialize_callback)(ZegoWhiteboardErrorCode error_code, void* user_context);

/**
 * 获取当前房间内白板列表的回调
 *
 * @param seq 调用zego_whiteboard_get_whiteboard_list返回的请求序号
 * @param error_code 错误码
 * @param whiteboard_id_list 房间内的白板列表
 * @param count 房间内的白板数量
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_get_whiteboard_list_callback)(ZegoSeq seq, ZegoWhiteboardErrorCode error_code, ZegoWhiteboardID* whiteboard_id_list, unsigned int count, void* user_context);

/**
 * 创建view的回调
 *
 * @param seq 调用 zego_whiteboard_create_view 返回的请求序号
 * @param error_code 错误码
 * @param view_id 白板viewID
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_create_view_callback)(ZegoSeq seq, ZegoWhiteboardErrorCode error_code, ZegoWhiteboardViewID view_id, void* user_context);

/**
 * 销毁view的回调
 *
 * @param seq 调用 zego_whiteboard_destroy_view 返回的请求序号
 * @param error_code 错误码
 * @param view_id 被销毁的白板viewID
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_destroy_view_callback)(ZegoSeq seq, ZegoWhiteboardErrorCode error_code, ZegoWhiteboardViewID view_id, void* user_context);

/**
 * 创建白板的回调
 *
 * @param seq 调用 zego_whiteboard_create_whiteboard 返回的请求序号
 * @param error_code 错误码
 * @param whiteboard_id_list 创建成功返回的白板列表
 * @param count 白板数量，创建Excel类型的白板时，返回的是Sheet的数量，因为一个Sheet对应一个白板，其他类型均为1
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_create_whiteboard_callback)(ZegoSeq seq, ZegoWhiteboardErrorCode error_code, ZegoWhiteboardID* whiteboard_id_list, int count, void* user_context);

/**
 * 销毁白板的回调
 *
 * @param seq 调用 zego_whiteboard_destroy_whiteboard 返回的请求序号
 * @param error_code 错误码
 * @param whiteboard_id 被销毁的白板
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_destroy_whiteboard_callback)(ZegoSeq seq, ZegoWhiteboardErrorCode error_code, ZegoWhiteboardID whiteboard_id, void* user_context);

/**
 * 房间内其他成员创建白板的通知
 *
 * @param whiteboard_id_list 其他成员创建的白板列表
 * @param count 白板数量
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_added_whiteboard_notify)(ZegoWhiteboardID* whiteboard_id_list, unsigned int count, void* user_context);

/**
 * 房间内其他成员销毁白板的通知
 *
 * @param whiteboard_id 被销毁的白板
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_removed_whiteboard_notify)(ZegoWhiteboardID whiteboard_id, void* user_context);

/**
 * 白板发生滚动的通知
 *
 * 本端的翻页、滚动和房间内其他成员的翻页、滚动都会触发此通知
 *
 * @param whiteboard_id 发生滚动的白板
 * @param horizontal_percent 滚动之后的水平百分比
 * @param vertical_percent 滚动之后的垂直百分比
 * @param page 滚动之后的当前页码
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_scroll_changed_notify)(ZegoWhiteboardID whiteboard_id, float horizontal_percent, float vertical_percent, unsigned int page, void* user_context);

/**
 * 文件上传转码进度通知
 *
 * @param seq 调用 zego_whiteboard_docs_upload 返回的请求序号
 * @param status 文件上传转码状态
 * @param rate 文件上传进度
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_docs_upload_progress_notify)(ZegoSeq seq, enum ZegoWhiteboardDocsUploadStatus status, float rate, void *user_context);

/**
 * 文件上传转码的回调
 *
 * @param seq 调用 zego_whiteboard_docs_upload 返回的请求序号
 * @param error_code 错误码
 * @param file_id 文件ID，创建白板时使用
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_docs_upload_callback)(ZegoSeq seq, ZegoWhiteboardErrorCode error_code, const char* file_id, void* user_context);

/**
 * 动态ppt发生滚动或者步数变更的通知
 *
 * @param whiteboard_id 发生滚动的白板
 * @param horizontal_percent 滚动后的水平百分比
 * @param vertical_percent 滚动后的垂直百分比
 * @param page 滚动后的当前页码
 * @param step 滚动后的当前页的当前步数
 * @param user_context 用户自定义数据
 */
typedef void(*zego_whiteboard_docs_step_changed_notify)(ZegoWhiteboardID whiteboard_id, float horizontal_percent, float vertical_percent, unsigned int page, unsigned int step, void* user_context);

#endif