#ifndef ZEGO_WHITEBOARD_DOCS_H
#define ZEGO_WHITEBOARD_DOCS_H

#include "zego-whiteboard-defines.h"

#ifdef __cplusplus
extern "C" {
#endif

	/**
	 * 上传文件并转码
	 *
	 * @param file_path 文件本地路径，utf8编码
	 * @param type 转码类型，动态ppt类型文件选择kZegoWhiteboardDocsRenderTypeVectorAndIMG，其他类型可选择kZegoWhiteboardDocsRenderTypeVectorAndIMG
	 * @param password 密码，暂未使用
	 *
	 * @return 请求序号
	 */
	ZEGOAPI ZegoSeq ZEGOCALL zego_whiteboard_docs_upload(const char* file_path, enum ZegoWhiteboardDocsRenderType type, const char* password);

	/**
	 * 注册上传文件进度通知
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_docs_reg_upload_progress_notify(zego_whiteboard_docs_upload_progress_notify fn, void *user_context);

	/**
	 * 注册上传文件结果回调
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_docs_reg_upload_callback(zego_whiteboard_docs_upload_callback fn, void* user_context);

	/**
	 * 获取指定ppt所有页的备注
	 *
	 * @param file_id 文件ID
	 *
	 * @return 所有页的备注，以“|”分割，note1|note2|note3|....
	 */
	ZEGOAPI const char* ZEGOCALL zego_whiteboard_docs_get_ppt_notes(const char* file_id);

	/**
	 * 获取指定文件所有页的缩略图
	 *
	 * @param file_id 文件ID
	 *
	 * @return 所有页的缩略图url，以“|”分割，url1|url2|url3|....
	 */
	ZEGOAPI const char* ZEGOCALL zego_whiteboard_docs_get_thumbnail_url(const char* file_id);

#ifdef __cplusplus
}
#endif
#endif